"""HTTP server for the dashboard."""

from __future__ import annotations


def serve(port: int = 8088) -> None:
    """Start HTTP server serving dashboard.html."""
    raise NotImplementedError("server.serve coming in Phase 2.")
